<?php
/*
Plugin Name: EME Sync Facebook Events
Plugin URI: http://www.e-dynamics.be/wordpress
Description: Sync Facebook Events to The Events Made Easy Plugin <a href="options-general.php?page=eme-sync-facebook-events%2Feme-sync-facebook-events.php">Settings</a> | <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=SMGDS4GLCYWNG&lc=BE&item_name=To%20support%20development%20of%20EME&currency_code=EUR&bn=PP%2dDonationsBF%3abtn_donateCC_LG%2egif%3aNonHosted">Donate</a>
Author: Franky Van Liedekerke
Version: 1.0.40
Author URI: https://www.e-dynamics.be
Text Domain: eme-sync-facebook-events
Domain Path: /langs
*/
 
/*  Copyright 2014-2017 Franky Van Liedekerke

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

// include needed if we run from cron (weirdly ...)
require_once( ABSPATH . 'wp-admin/includes/image.php' );
// for media upload
require_once( ABSPATH . 'wp-admin/includes/file.php' );
require_once( ABSPATH . 'wp-admin/includes/media.php' );

// include required files form Facebook SDK
//require __DIR__ . '/Facebook/facebook-php-sdk-v4-4.0.23/autoload.php';
require __DIR__ . '/Facebook/facebook-php-sdk-v5.0.0/autoload.php';

register_activation_hook(__FILE__,'activate_eme_sfe');
register_deactivation_hook(__FILE__,'deactivate_eme_sfe');
function activate_eme_sfe() { wp_schedule_event(time(), 'daily', 'eme_sfe_execute_sync'); }
function deactivate_eme_sfe() { wp_clear_scheduled_hook('eme_sfe_execute_sync'); }
add_action('eme_sfe_execute_sync', 'eme_sfe_process_events_silent');
add_action('init','eme_sfe_load_textdomain');

$eme_timezone = get_option('timezone_string');
if (!$eme_timezone) {
   $offset = get_option('gmt_offset');
   $eme_timezone = timezone_name_from_abbr(null, $offset * 3600, false);
   if($eme_timezone === false) $eme_timezone = timezone_name_from_abbr(null, $offset * 3600, true);
}
$eme_timezone_obj = new DateTimeZone($eme_timezone);

function eme_sfe_load_textdomain() {
   $thisDir = dirname( plugin_basename( __FILE__ ) );
   load_plugin_textdomain('eme-sync-facebook-events', false, $thisDir.'/langs');
}

function update_schedule($eme_sfe_frequency) {
   wp_clear_scheduled_hook('eme_sfe_execute_sync');
   if ($eme_sfe_frequency!="none") {
      wp_schedule_event(time(), $eme_sfe_frequency, 'eme_sfe_execute_sync');
   }
}

function eme_sfe_media_sideload_image($url,$event_name) {
   // from media_sideload_image
   if ( ! empty($url) ) {
      // Download file to temp location
      $tmp = download_url( $url );

      // Set variables for storage
      // fix file filename for query strings
      preg_match( '/[^\?]+\.(jpe?g|jpe|gif|png)\b/i', $url, $matches );
      $file_array['name'] = basename($matches[0]);
      $file_array['tmp_name'] = $tmp;

      // If error storing temporarily, unlink
      if ( is_wp_error( $tmp ) ) {
         @unlink($file_array['tmp_name']);
         $file_array['tmp_name'] = '';
         return false;
      }

      // do the validation and storage stuff
      $post_id=0;
      $desc="Cover image for EME event '$event_name'";
      $id = media_handle_sideload( $file_array, $post_id, $desc );
      // If error storing permanently, unlink
      if ( is_wp_error($id) ) {
         @unlink($file_array['tmp_name']);
         return false;
      }

      $src = wp_get_attachment_url( $id );
      return array(0=>$id,1=>$src);
   } else {
      return false;
   }
}

function eme_sfe_add_page() { add_options_page('EME Sync FB Events', 'EME Sync FB Events', 'activate_plugins', __FILE__, 'eme_sfe_options_page'); }
add_action('admin_menu', 'eme_sfe_add_page');

function eme_sfe_process_events_silent() {
	eme_sfe_process_events(0);
}
function eme_sfe_process_events($show_results=1) {
   // Get option values
   $eme_sfe_api_key = get_option('eme_sfe_api_key');
   $eme_sfe_api_secret = get_option('eme_sfe_api_secret');
   $eme_sfe_api_uids = get_option('eme_sfe_api_uids');
   if (!$eme_sfe_api_uids) return;
   // remove empty array values
   $eme_sfe_api_uids = array_filter($eme_sfe_api_uids,'strlen');
   $fb_access_token = get_option('eme_sfe_fb_access_token');
   $length = get_option('eme_sfe_length');
   if ($length>0) {
	   $offset = get_option('eme_sfe_offset');
	   if (!$offset) $offset=0;
	   $arr_to_sync=array_slice($eme_sfe_api_uids,$offset,$length);
	   $offset+=$length;
	   if ($offset>=count($eme_sfe_api_uids)) $offset=0;
	   update_option('eme_sfe_offset',$offset);
	   eme_sfe_sync_events($eme_sfe_api_key, $eme_sfe_api_secret, $arr_to_sync,$fb_access_token,$show_results);
   } else {
	   eme_sfe_sync_events($eme_sfe_api_key, $eme_sfe_api_secret, $eme_sfe_api_uids,$fb_access_token,$show_results);
   }
}

function eme_sfe_check_session($eme_sfe_api_key,$eme_sfe_api_secret,$facebook_session) {
   global $eme_timezone_obj;
   $eme_time_obj_now = new DateTime(null,$eme_timezone_obj);
	if (!$facebook_session->isExpired() && $facebook_session->isLongLived()) {
		$expiry_date=$facebook_session->getExpiresAt();
		// check expiry date: if less than 10 days, renew it
		$date_diff = date_diff($eme_time_obj_now,$expiry_date,true);
		if ($date_diff->format('%a') <=10) {
			$fb = new Facebook\Facebook([
				'app_id' => $eme_sfe_api_key,
				'app_secret' => $eme_sfe_api_secret,
				'default_graph_version' => 'v2.8',
			]);
			$fb->setDefaultAccessToken($facebook_session);
			$oAuth2Client = $fb->getOAuth2Client();
			$code = $oAuth2Client->getCodeFromLongLivedAccessToken($facebook_session);
			$facebook_session = $oAuth2Client->getAccessTokenFromCode($code);
			update_option('eme_sfe_fb_access_token',$facebook_session);
		}
	}
        //if (!$facebook_session) {
	//	$facebook_session = FacebookSession::newAppSession();
	//	update_option('facebook_session',$facebook_session);
	//}
	return $facebook_session;
}

function eme_sfe_sync_event($eme_sfe_api_key,$eme_sfe_api_secret,$event_id,$facebook_session) {
   global $eme_timezone_obj;
   if (empty($eme_sfe_api_key) || empty($eme_sfe_api_secret) || empty($event_id) || empty($facebook_session))
      return false;

   $facebook_session = eme_sfe_check_session($eme_sfe_api_key,$eme_sfe_api_secret,$facebook_session);
   $fb = new Facebook\Facebook([
        'app_id' => $eme_sfe_api_key,
        'app_secret' => $eme_sfe_api_secret,
        'default_graph_version' => 'v2.8',
        ]);
   $fb->setDefaultAccessToken($facebook_session);

   // accept both event id and https://www.facebook.com/events/<event_id>
   $event_id=preg_replace('/^.*facebook.com\/events\//','',$event_id);
   $event_id=preg_replace('/\/.*$/','',$event_id);

   // the following works, but doesn't return the cover (api bug?), so we specify the fields we want
   //$event = (new FacebookRequest( $facebook_session, 'GET', '/'.$event_id))->execute()->getGraphObject()->asArray();
   //$fields=array("fields"=>"id,name,location,venue,start_time,end_time,is_date_only,description,cover");
   try {
	   // Requires the "read_stream" permission
	   $fields="id,name,place,start_time,end_time,event_times,description,ticket_uri,cover,picture{url,height,width}";
	   $request = $fb->get('/'.$event_id.'?fields='.$fields);
   } catch(Facebook\Exceptions\FacebookResponseException $e) {
	   // When Graph returns an error
	   echo 'Graph returned an error: ' . $e->getMessage();
	   exit;
   } catch(Facebook\Exceptions\FacebookSDKException $e) {
	   // When validation fails or other local issues
	   echo 'Facebook SDK returned an error: ' . $e->getMessage();
	   exit;
   }
   $event = $request->getGraphNode()->asArray();
   if (isset($event['cover']) && !empty($event['cover']) && isset($event['cover']['source'])) {
      $event['event_picture_url']=$event['cover']['source'];
   } elseif (isset($event['picture']) && !empty($event['picture']) && isset($event['picture']['url'])) {
      $event['event_picture_url']=$event['picture']['url'];
   }

   $eme_time_obj_now = new DateTime(null,$eme_timezone_obj);
   if (isset($event['event_times'])) {
	   foreach ($event['event_times'] as $occ) {
		   $event['start_time']=$occ['start_time'];
		   $event['id']=$occ['id'];
		   if (isset($occ['end_time']))
			   $event['end_time']=$occ['end_time'];
		   if ($event['start_time'] > $eme_time_obj_now)
			   eme_sfe_send_event($event);
	   }
   } elseif ($event['start_time'] > $eme_time_obj_now) {
	   eme_sfe_send_event($event);
   }
}

function eme_sfe_sync_events($eme_sfe_api_key, $eme_sfe_api_secret, $eme_sfe_uids, $facebook_session,$show_results=1) {
   global $eme_timezone_obj;
   if (empty($eme_sfe_api_key) || empty($eme_sfe_api_secret) || empty($eme_sfe_uids) || empty($facebook_session))
      return false;

   $facebook_session = eme_sfe_check_session($eme_sfe_api_key,$eme_sfe_api_secret,$facebook_session);
   $fb = new Facebook\Facebook([
        'app_id' => $eme_sfe_api_key,
        'app_secret' => $eme_sfe_api_secret,
        'default_graph_version' => 'v2.8',
        ]);
   $fb->setDefaultAccessToken($facebook_session);

   $ret = array();
   $eme_time_obj_now = new DateTime(null,$eme_timezone_obj);
   foreach ($eme_sfe_uids as $key => $value) {
      // accept both pagename and https://www.facebook.com/<pagename>
      $value=preg_replace('/^.*facebook.com\//','',$value);
      $value=preg_replace('/\/.*$/','',$value);
      if ($value!='') {
         if ($show_results) echo "<br />Trying to sync events for $value";
	 try {
	      // Requires the "read_stream" permission
              $fields="id,name,place,start_time,end_time,event_times,description,ticket_uri,cover,picture{url,height,width}";
	      $request = $fb->get('/'.$value.'/events?fields='.$fields);
	 } catch(Facebook\Exceptions\FacebookResponseException $e) {
	      // When Graph returns an error
	      if ($show_results) echo 'Graph returned an error: ' . $e->getMessage();
	      exit;
	 } catch(Facebook\Exceptions\FacebookSDKException $e) {
	      // When validation fails or other local issues
	      if ($show_results) echo 'Facebook SDK returned an error: ' . $e->getMessage();
	      exit;
	 }
         $response = $request->getGraphEdge(); // this gets a page of results
         do {
            foreach ($response as $graphnode) {
               $event=$graphnode->asArray();
               if (isset($event['cover']) && !empty($event['cover']) && isset($event['cover']['source'])) {
                  $event['event_picture_url']=$event['cover']['source'];
               } elseif (isset($event['picture']) && !empty($event['picture']) && isset($event['picture']['url'])) {
                  $event['event_picture_url']=$event['picture']['url'];
               }

	       if (isset($event['event_times'])) {
		       foreach ($event['event_times'] as $occ) {
			       $event['id']=$occ['id'];
			       $event['start_time']=$occ['start_time'];
			       if (isset($occ['end_time']))
				       $event['end_time']=$occ['end_time'];
			       if ($event['start_time'] > $eme_time_obj_now)
				       eme_sfe_send_event($event,$show_results);
		       }
	       } elseif ($event['start_time'] > $eme_time_obj_now) {
		       eme_sfe_send_event($event,$show_results);
	       }
            }
         } while ($response = $fb->next($response));
      }
   }
}

function eme_sfe_check_image_id($id) {
   global $wpdb;
   // returns false if the ID doesn't exist or is not an attachment
   $sql = $wpdb->prepare("SELECT id FROM wp_posts WHERE id = %d AND post_type='attachment'",$id);
   return $wpdb->get_var($sql);
}

function eme_sfe_check_event_fbid($id) {
   global $wpdb;
   $table_name = $wpdb->prefix . EVENTS_TBNAME;
   $sql = $wpdb->prepare("SELECT event_id FROM $table_name WHERE event_external_ref = %s","fb_".$id);
   return $wpdb->get_var($sql);
}

function eme_sfe_check_location_fbid($id) {
   global $wpdb;
   $table_name = $wpdb->prefix . LOCATIONS_TBNAME;
   $sql = $wpdb->prepare("SELECT location_id FROM $table_name WHERE location_external_ref = %s","fb_".$id);
   return $wpdb->get_var($sql);
}

function eme_sfe_check_location_coord($lat,$long) {
   global $wpdb;
   $table_name = $wpdb->prefix . LOCATIONS_TBNAME;
   $sql = $wpdb->prepare("SELECT location_id FROM $table_name WHERE location_latitude = %s AND location_longitude = %s",$lat,$long);
   return $wpdb->get_var($sql);
}

function eme_sfe_check_location_name_address($location) {
   global $wpdb;
   $table_name = $wpdb->prefix . LOCATIONS_TBNAME;
   $sql=$wpdb->prepare("SELECT location_id FROM $table_name WHERE location_name = %s AND location_address1 = %s AND location_address2 = %s AND location_city = %s AND location_state = %s AND location_zip = %s AND location_country = %s LIMIT 1", stripcslashes($location['location_name']), stripcslashes($location['location_address1']), stripcslashes($location['location_address2']),stripcslashes($location['location_city']),stripcslashes($location['location_state']),stripcslashes($location['location_zip']),stripcslashes($location['location_country']));
   return $wpdb->get_var($sql);
}

function eme_sfe_send_event($fb_event,$show_results=1) {
   global $eme_timezone_obj;
   $add_location_info=0;
   if (isset($fb_event['place'])) {
         $add_location_info=1;

         $location = eme_new_location();
	 if (isset($fb_event['place']['name']))
            $location['location_name'] = $fb_event['place']['name'];
	 if (isset($fb_event['place']['location'])) {
            if (isset($fb_event['place']['location']['street']))
               $location['location_address1'] = $fb_event['place']['location']['street'];
            if (isset($fb_event['place']['location']['zip']))
               $location['location_zip'] = $fb_event['place']['location']['zip'];
            if (isset($fb_event['place']['location']['city']))
               $location['location_city']=$fb_event['place']['location']['city'];
            if (isset($fb_event['place']['location']['state']))
               $location['location_state']=$fb_event['place']['location']['state'];
            if (isset($fb_event['place']['location']['country']))
               $location['location_country']=$fb_event['place']['location']['country'];
	 }
            
         if (isset($fb_event['place']['id']))
            $location_id=eme_sfe_check_location_fbid($fb_event['place']['id']);
         else
            $location_id=0;
         if (!$location_id && get_option('eme_sfe_use_loc_coord') && isset($fb_event['place']['location']['latitude']) && isset($fb_event['place']['location']['longitude']))
            $location_id=eme_sfe_check_location_coord($fb_event['place']['location']['latitude'],$fb_event['place']['location']['longitude']);
         if (!$location_id && get_option('eme_sfe_use_loc_name'))
            $location_id=eme_sfe_check_location_name_address($location);

         if ($location_id)
            $location = eme_get_location($location_id);
            
         if (isset($fb_event['place']['location']['latitude']))
            $location['location_latitude'] = $fb_event['place']['location']['latitude'];
         if (isset($fb_event['place']['location']['longitude']))
            $location['location_longitude'] = $fb_event['place']['location']['longitude'];

         $location['location_description'] = '';
         if (isset($fb_event['place']['id']))
            $location['location_external_ref'] = 'fb_'.$fb_event['place']['id'];

         $location = eme_sanitize_location($location);
         if ($location_id) {
            if (get_option('eme_sfe_skip_synced')) {
               if ($show_results) echo "<br />Skipping already synchronized location: $location_id";
            } else {
               eme_update_location($location,$location_id);
               if ($show_results) echo "<br />Updated location: $location_id";
            }
         } else {
            $location = eme_insert_location($location);
            $location_id = $location['location_id'];
            if ($show_results) echo "<br />Inserted location: $location_id";
         }
      }

      $event_id=eme_sfe_check_event_fbid($fb_event['id']);
      if ($event_id)
         $event = eme_get_event($event_id);
      else
         $event = eme_new_event();

      $event['event_name']=$fb_event['name'];
      $event['event_start_date']=$fb_event['start_time']->format("Y-m-d");

//      if (!preg_match('/T/',$fb_event['start_time'])) {
 //        $event['event_start_time']="00:00:00";
  //       $event['event_properties']['all_day']=1;
   //   } else {
         $event['event_start_time']=$fb_event['start_time']->format("H:i:s");
         $event['event_properties']['all_day']=0;
    //  }

      if (isset($fb_event['end_time'])) {
         $event['event_end_date']=$fb_event['end_time']->format("Y-m-d");
         $event['event_end_time']=$fb_event['end_time']->format("H:i:s");
      }

      $event['event_status'] = get_option('eme_sfe_event_initial_state');
      // fb makes links clickable, but not via their api, so we'll do it here
      if (isset($fb_event['description']))
         $event['event_notes'] = make_clickable($fb_event['description']);
      $event['event_external_ref'] = 'fb_'.$fb_event['id'];
      // let's store the optional ticket uri so people could use it as an attribute
      if (isset($fb_event['ticket_uri']))
         $event['event_attributes']['fb_ticket_uri'] = $fb_event['ticket_uri'];
      if ($add_location_info)
         $event['location_id']=$location_id;
      if ($event_id) {
         if (get_option('eme_sfe_skip_synced')) {
            if ($show_results) echo "<br />Skipping already synchronized event: $event_id";
         } else {
            if (isset($fb_event['event_picture_url'])) {
	       if (get_option('eme_sfe_download_img')) {
		       // facebook picture url can contain query args, se we need to remove them
		       $fb_event_picture_url_arr=parse_url($fb_event['event_picture_url']);
		       $fb_event_picture_path=basename($fb_event_picture_url_arr['path']);
		       $event_image_path=basename($event['event_image_url']);
		       // we'll store the fb picture url as an attribute, since wp could change the name of the uploaded
		       // image if it already exists and then the image would be uploaded upon each sync ...
		       if (!isset($event['event_attributes']['fb_event_picture_path']) ||
			       (isset($event['event_attributes']['fb_event_picture_path']) && $event['event_attributes']['fb_event_picture_path'] != $fb_event_picture_path)) {
			       // only upload if needed
			       if (($fb_event_picture_path != $event_image_path) ||
				       (!empty($event['event_image_id']) && !eme_sfe_check_image_id($event['event_image_id']))) {
				       $res=eme_sfe_media_sideload_image($fb_event['event_picture_url'],$fb_event['name']);
				       if ($res && is_array($res)) {
					       $event['event_image_id']=$res[0];
					       $event['event_image_url']=$res[1];
				       }
			       }
			       $event['event_attributes']['fb_event_picture_path']=$fb_event_picture_path;
		       }
	       } else {
		       $event['event_image_url']=$fb_event['event_picture_url'];
	       }
            }
            $event = eme_sanitize_event($event);
            eme_db_update_event($event,$event_id);
            if ($show_results) echo "<br />Updated event: ".$event_id;
         }
      } else {
         if (isset($fb_event['event_picture_url'])) {
	       if (get_option('eme_sfe_download_img')) {
		       $res=eme_sfe_media_sideload_image($fb_event['event_picture_url'],$fb_event['name']);
		       if ($res && is_array($res)) {
			       $event['event_image_id']=$res[0];
			       $event['event_image_url']=$res[1];
		       }
	       } else {
		       $event['event_image_url']=$fb_event['event_picture_url'];
	       }
         }
         $event = eme_sanitize_event($event);
         $event_id=eme_db_insert_event($event);
         if ($show_results) echo "<br />Inserted event: ".$event_id;
      }
}

function eme_sfe_options_page() {
   $eme_sfe_frequencies=array('daily'=>__("Daily",'eme-sync-facebook-events'),"twicedaily"=>__("Twice Daily",'eme-sync-facebook-events'),"hourly"=>__("Hourly",'eme-sync-facebook-events'),"none"=>__("None",'eme-sync-facebook-events'));
?>
   <div id="fb-root"></div>
   <script>
   window.fbAsyncInit = function() {
      // init the FB JS SDK
      FB.init({
         appId      : '<?php echo get_option("eme_sfe_api_key");?>',// App ID from the app dashboard
         cookie     : true,  // let's use a cookie afterwards for the php scripts
         status     : true,  // Check Facebook Login status
         xfbml      : true,   // Look for social plugins on the page
	 version    : 'v2.8'
      });
   };

   // Load the SDK asynchronously
   (function(d, s, id){
       var js, fjs = d.getElementsByTagName(s)[0];
       if (d.getElementById(id)) {return;}
       js = d.createElement(s); js.id = id;
       js.src = "//connect.facebook.net/en_US/all.js";
       fjs.parentNode.insertBefore(js, fjs);
    }(document, 'script', 'facebook-jssdk'));
   </script>

<?php
   // Get option values
   $eme_sfe_api_key = get_option('eme_sfe_api_key');
   $eme_sfe_api_secret = get_option('eme_sfe_api_secret');
   $eme_sfe_api_uid = get_option('eme_sfe_api_uid');
   $eme_sfe_api_uids = get_option('eme_sfe_api_uids');
   if (!$eme_sfe_api_uids)
      $eme_sfe_api_uids = array();
   // remove empty array values
   $eme_sfe_api_uids = array_filter($eme_sfe_api_uids,'strlen');
   $eme_sfe_frequency = get_option('eme_sfe_frequency');
   if (!$eme_sfe_frequency) $eme_sfe_frequency="none";
   if (!array_key_exists($eme_sfe_frequency,$eme_sfe_frequencies))
	   $eme_sfe_frequency="none";

   $msg="";
   $js_session = false;
   if (!empty($eme_sfe_api_key) && !empty($eme_sfe_api_secret)) {
      $fb = new Facebook\Facebook([
        'app_id' => $eme_sfe_api_key,
        'app_secret' => $eme_sfe_api_secret,
        'default_graph_version' => 'v2.8',
        ]);

      $helper = $fb->getJavaScriptHelper();
      try {
         $js_session = $helper->getAccessToken();
      } catch(Facebook\Exceptions\FacebookResponseException $e) {
         // When Facebook returns an error
         $mgs = sprintf(__("Facebook login error: %s",'eme-sync-facebook-events'),$e);
      } catch(Facebook\Exceptions\FacebookSDKException $e) {
         // When validation fails or other local issues
         $mgs = sprintf(__("Facebook validation error: %s",'eme-sync-facebook-events'),$e);
      }
   }

   if( !empty($_POST['clear_session']) ) {
	   delete_option('eme_sfe_fb_access_token');
	   $fb_access_token = false;
	   $force_update=true;
   } else {
	   $fb_access_token = get_option('eme_sfe_fb_access_token');
	   $force_update=false;
   }

   if (!$fb_access_token || $fb_access_token->isExpired()) {
      if (!$js_session) {
         $msg = __("No valid Facebook session for this app detected, please logout/login using the button.",'eme-sync-facebook-events');
      } else {
         // The OAuth 2.0 client handler helps us manage access tokens
         $oAuth2Client = $fb->getOAuth2Client();
	 // Get the access token metadata from /debug_token
	 //$tokenMetadata = $oAuth2Client->debugToken($accessToken);
	 // Validation (these will throw FacebookSDKException's when they fail)
	 //$tokenMetadata->validateAppId($eme_sfe_api_key);
	 //$tokenMetadata->validateExpiration();
	 if (!$js_session->isLongLived()) {
		 // Exchanges a short-lived access token for a long-lived one
		 try {
			 $fb_access_token = $oAuth2Client->getLongLivedAccessToken($js_session);
		 } catch (Facebook\Exceptions\FacebookSDKException $e) {
			 echo "<p>Error getting long-lived access token: " . $helper->getMessage() . "</p>\n\n";
			 exit;
		 }
		 update_option('eme_sfe_fb_access_token',$fb_access_token);
	 }
      }
   }

   $events=false;
   // Get new updated option values, and save them
   if( !empty($_POST['update']) || $force_update ) {

      $eme_sfe_api_key = sanitize_text_field($_POST['eme_sfe_api_key']);
      update_option('eme_sfe_api_key', $eme_sfe_api_key);

      $eme_sfe_api_secret = sanitize_text_field($_POST['eme_sfe_api_secret']);
      update_option('eme_sfe_api_secret', $eme_sfe_api_secret);

      $eme_sfe_frequency = sanitize_text_field($_POST['eme_sfe_frequency']);
      if (!array_key_exists($eme_sfe_frequency,$eme_sfe_frequencies))
	      $eme_sfe_frequency="none";
      update_option('eme_sfe_frequency', $eme_sfe_frequency);

      $eme_sfe_length = intval($_POST['eme_sfe_length']);
      update_option('eme_sfe_length', $eme_sfe_length);

      $eme_sfe_event_initial_state = sanitize_text_field($_POST['eme_sfe_event_initial_state']);
      if (!array_key_exists($eme_sfe_event_initial_state,eme_status_array()))
	      $eme_sfe_event_initial_state=5; //draft
      update_option('eme_sfe_event_initial_state', $eme_sfe_event_initial_state);

      $eme_sfe_download_img = intval($_POST['eme_sfe_download_img']);
      update_option('eme_sfe_download_img', $eme_sfe_download_img);

      $eme_sfe_skip_synced = intval($_POST['eme_sfe_skip_synced']);
      update_option('eme_sfe_skip_synced', $eme_sfe_skip_synced);

      $eme_sfe_use_loc_coord = intval($_POST['eme_sfe_use_loc_coord']);
      update_option('eme_sfe_use_loc_coord', $eme_sfe_use_loc_coord);

      $eme_sfe_use_loc_name = intval($_POST['eme_sfe_use_loc_name']);
      update_option('eme_sfe_use_loc_name', $eme_sfe_use_loc_name);

      if (!empty($msg)) {
      ?>
         <div id="message" class="error fade"><p><strong><?php echo "...".$msg."..."; ?></strong></p></div>
      <?php
      }
      if ($fb_access_token) {
         //eme_sfe_sync_events($eme_sfe_api_key, $eme_sfe_api_secret, $eme_sfe_api_uids, $fb_access_token);
	 eme_sfe_process_events();
	 $expiry_date=$fb_access_token->getExpiresAt();
         if (is_null($expiry_date))
            $expire_msg = __("FB session apparently never expires",'eme-sync-facebook-events');
         else
            $expire_msg = sprintf(__("FB session will expire at %s",'eme-sync-facebook-events'),$expiry_date->format('Y-m-d H:i:s'));

      ?>
         <div style="margin-top:20px;font-size:14px;color:#444;border:1px solid #999;padding:15px;width:95%;font-face:couriernew;">
         <span style="color:red;"><?php _e('Events Made Easy updated with current Facebook events.','eme-sync-facebook-events'); ?></span><br /><br />
         <span style="color:red;"><?php print $expire_msg; ?></span><br /><br />
         </div>
      <?php
      }
      update_schedule($eme_sfe_frequency);

   } elseif( !empty($_POST['add-event-id']) ) {
      $event_id = sanitize_text_field(trim($_POST['eme_sfe_event_id']));
      if (!empty($msg)) {
      ?>
      <div id="message" class="updated fade"><p><strong><?php echo $msg; ?></strong></p></div>
      <?php
      }
      if ($fb_access_token) {
         eme_sfe_sync_event($eme_sfe_api_key, $eme_sfe_api_secret, $event_id, $fb_access_token);
	 $expiry_date=$fb_access_token->getExpiresAt();
         if (is_null($expiry_date))
            $expire_msg = __("FB session apparently never expires",'eme-sync-facebook-events');
         else
            $expire_msg = sprintf(__("FB session will expire at %s",'eme-sync-facebook-events'),$expiry_date->format('Y-m-d H:i:s'));
      ?>
         <div style="margin-top:20px;font-size:14px;color:#444;border:1px solid #999;padding:15px;width:95%;font-face:couriernew;">
         <span style="color:red;"><?php _e('Synced new facebook event with Events Made Easy.','eme-sync-facebook-events'); ?></span><br /><br />
         <span style="color:red;"><?php print $expire_msg; ?></span><br /><br />
         </div>
      <?php
      }

   } elseif( !empty($_POST['add-uid']) ) {
      if(!in_array($_POST['eme_sfe_api_uid'], $eme_sfe_api_uids)) {
         $eme_sfe_api_uids[] = sanitize_text_field(trim($_POST['eme_sfe_api_uid']));
         update_option('eme_sfe_api_uids', $eme_sfe_api_uids);
      }

   } elseif( !empty($_GET['r']) ) {
      $removal_param = sanitize_text_field(urldecode($_GET['r']));
      foreach ($eme_sfe_api_uids as $key => $value)
         if($eme_sfe_api_uids[$key] == $removal_param)
            unset($eme_sfe_api_uids[$key]);
      update_option('eme_sfe_api_uids', $eme_sfe_api_uids);
   }	
   $this_page_url=$_SERVER['REQUEST_URI'];

   if (!function_exists("eme_options_input_text") || !function_exists("eme_options_input_text")) {
      ?>
      <div id="message" class="error"><p><?php _e("This plugin requires 'Events Made Easy' to be installed.",'eme-sync-facebook-events'); ?> </p></div>
      <?php
      return;
   }

   ?>
   <div class="wrap">
   <br /><div class="icon32" id="icon-plugins"><br /></div>
   <h2 style="margin-bottom:10px;">Events Made Easy Sync Facebook Events</h2>
   <form method="post" action="">
   <table class="form-table"> 
   <?php
   eme_options_input_text ( __('Facebook App ID', 'eme-sync-facebook-events' ), 'eme_sfe_api_key', '');
   eme_options_input_text ( __('Facebook App Secret', 'eme-sync-facebook-events' ), 'eme_sfe_api_secret', '<input type="submit" value="'.__("Clear session cache",'eme-sync-facebook-events').'" class="button-secondary" name="clear_session" /><br />'.__("Clear stored session cache.",'eme-sync-facebook-events'));
   eme_options_select (__('Update Frequency','eme-sync-facebook-events'), 'eme_sfe_frequency', $eme_sfe_frequencies, '');
   eme_options_select (__('State for new event','eme-sync-facebook-events'), 'eme_sfe_event_initial_state', eme_status_array(), '');
   eme_options_radio_binary (__('Use coordinates for locations','eme-sync-facebook-events'), 'eme_sfe_use_loc_coord', __("Normally, the facebook location ID is used to check wether a location has been synchronized or not. Sometimes you want to use own locations with the same coordinates (latitude and longitude), so select 'Yes' to check for matching locations using coordinates.",'eme-sync-facebook-events'));
   eme_options_radio_binary (__('Use name for locations','eme-sync-facebook-events'), 'eme_sfe_use_loc_name', __("Normally, the facebook location ID is used to check wether a location has been synchronized or not. Sometimes you want to use own locations with the same name and address, so select 'Yes' to check for matching locations using just the name and address info.",'eme-sync-facebook-events'));
   eme_options_radio_binary (__('Skip synced events and locations','eme-sync-facebook-events'), 'eme_sfe_skip_synced', __("Select 'Yes' to skip already synchronized events and locations, otherwise these will be overwritten with every sync",'eme-sync-facebook-events'));
   eme_options_radio_binary (__('Download event cover image','eme-sync-facebook-events'), 'eme_sfe_download_img', __("Select 'Yes' to download the event cover image and serve it locally, otherwise default the location to the facebook image will be used.",'eme-sync-facebook-events'));
   eme_options_input_int ( __('Number of facebook pages to sync in 1 go','eme-sync-facebook-events'), 'eme_sfe_length', __("If you have a large number of facebook pages to sync, you might run into a PHP timeout. If this happens, set here a non-zero number of facebook pages you want to sync in 1 go. Everytime you save the options or the cron runs, the next batch will then be synced.",'eme-sync-facebook-events'));
   eme_options_input_text ( __('Add Facebook Page', 'eme-sync-facebook-events' ).'<br /><fb:login-button id="fb-login-button" width="200" autologoutlink="true" scope="user_events" max-rows="1"></fb:login-button>', 'eme_sfe_api_uid', '<input type="submit" value="Add" class="button-secondary" name="add-uid" /><br />'.__("Can be a Facebook Page name like 'webtrends' or the Facebook Page ID for it.",'eme-sync-facebook-events'));
   ?>
   <tr><td style="vertical-align:top;"></td><td>
   <?php
   foreach ($eme_sfe_api_uids as $value) {
      if ($value!='')
         echo '&nbsp;&nbsp;'.$value.'&nbsp;&nbsp;<a href="'.add_query_arg(array('r'=>rawurlencode($value)),$this_page_url).'">remove</a><br />';
      }
   ?>
   </td></tr>
   <?php
   eme_options_input_text ( __('Import Single Event', 'eme-sync-facebook-events'), 'eme_sfe_event_id', '<input type="submit" value="'.__("Import Single",'eme-sync-facebook-events').'" class="button-secondary" name="add-event-id" /><br />'.__("Directly import a single Facebook event. Either the Facebook url or just the Facebook event id is sufficient.",'eme-sync-facebook-events'));
   ?>
   <tr><td colspan="2"><input type="submit" value="<?php _e('Update','eme-sync-facebook-events'); ?>" class="button-primary" name="update" /></td></tr>
   </table>
   </form>
   </div>
<?php
}
?>
